@@ -1,5 +1,5 @@
 import codecs
-from io import BytesIO
+from io import BytesIO, StringIO
 import distutils.dir_util
 import logging
 import math
@@ -277,6 +277,24 @@ def checkShiftJIS(first, second):
     return status
 
 
+def openSection(file, filestart=1, fileend=10):
+    if not os.path.isfile(file.format("")):
+        if os.path.isfile(file.format(str(filestart))):
+            section = StringIO()
+            for i in range(filestart, fileend + 1):
+                sectionfile = file.format(str(i))
+                if os.path.isfile(sectionfile):
+                    with codecs.open(sectionfile, "r", "utf-8") as f:
+                        section.write(f.read())
+                else:
+                    break
+            return section
+        else:
+            return None
+    else:
+        return codecs.open(file.format(""), "r", "utf-8")
+
+
 def getSection(f, title, comment="#", fixchars=[]):
     ret = {}
     found = title == ""
@@ -462,6 +480,113 @@ def writeEncodedString(f, s, maxlen=0, encoding="shift_jis"):
     return i
 
 
+def extractBinaryStrings(infile, binranges, func=detectEncodedString, encoding="shift_jis"):
+    strings = []
+    positions = []
+    insize = os.path.getsize(infile)
+    with Stream(infile, "rb") as f:
+        for binrange in binranges:
+            f.seek(binrange[0])
+            while f.tell() < binrange[1] and f.tell() < insize - 2:
+                pos = f.tell()
+                check = func(f, encoding)
+                if check != "":
+                    if check not in strings:
+                        logDebug("Found string at", pos)
+                        strings.append(check)
+                        positions.append([pos])
+                    else:
+                        positions[strings.index(check)].append(pos)
+                    pos = f.tell() - 1
+                f.seek(pos + 1)
+    return strings, positions
+
+
+def repackBinaryStrings(section, infile, outfile, binranges, freeranges=None, detectFunc=detectEncodedString, writeFunc=writeEncodedString, encoding="shift_jis", pointerstart=0):
+    insize = os.path.getsize(infile)
+    with Stream(infile, "rb") as fi:
+        if freeranges is not None:
+            allbin = fi.read()
+            strpointers = {}
+            freeranges = [list(x) for x in freeranges]
+        with Stream(outfile, "r+b") as fo:
+            for binrange in binranges:
+                fi.seek(binrange[0])
+                while fi.tell() < binrange[1] and fi.tell() < insize - 2:
+                    pos = fi.tell()
+                    check = detectFunc(fi, encoding)
+                    if check != "":
+                        if check in section and section[check][0] != "":
+                            newsjis = section[check][0]
+                            if len(section[check]) > 1:
+                                section[check].pop(0)
+                            if newsjis == "!":
+                                newsjis = ""
+                            newsjislog = newsjis.encode("ascii", "ignore")
+                            logDebug("Replacing string at", pos, "with", newsjislog)
+                            fo.seek(pos)
+                            endpos = fi.tell() - 1
+                            newlen = writeFunc(fo, newsjis, endpos - pos + 1, encoding)
+                            fo.seek(-1, 1)
+                            if fo.readByte() != 0:
+                                fo.writeZero(1)
+                            if newlen < 0:
+                                if freeranges is None or pointerstart == 0:
+                                    logError("String", newsjislog, "is too long.")
+                                else:
+                                    # Add this to the freeranges
+                                    freeranges.append([pos, endpos])
+                                    logDebug("Adding new freerage", pos, endpos)
+                                    range = None
+                                    rangelen = 0
+                                    for c in newsjis:
+                                        rangelen += 1 if ord(c) < 256 else 2
+                                    for freerange in freeranges:
+                                        if freerange[1] - freerange[0] > rangelen:
+                                            range = freerange
+                                            break
+                                    if range is None and newsjis not in strpointers:
+                                        logError("No more room! Skipping", newsjislog, "...")
+                                        freeranges.pop()
+                                    else:
+                                        # Write the string in a new portion of the rom
+                                        if newsjis in strpointers:
+                                            newpointer = strpointers[newsjis]
+                                        else:
+                                            logDebug("No room for the string", newsjislog, ", redirecting to", toHex(range[0]))
+                                            fo.seek(range[0])
+                                            writeFunc(fo, newsjis, 0, encoding)
+                                            fo.seek(-1, 1)
+                                            if fo.readByte() != 0:
+                                                fo.writeZero(1)
+                                            newpointer = pointerstart + range[0]
+                                            range[0] = fo.tell()
+                                            strpointers[newsjis] = newpointer
+                                        # Search and replace the old pointer
+                                        pointer = pointerstart + pos
+                                        pointersearch = struct.pack("<I", pointer)
+                                        index = 0
+                                        logDebug("Searching for pointer", toHex(pointer))
+                                        foundone = False
+                                        while index < len(allbin):
+                                            index = allbin.find(pointersearch, index)
+                                            if index < 0:
+                                                break
+                                            foundone = True
+                                            logDebug("Replaced pointer at", str(index))
+                                            fo.seek(index)
+                                            fo.writeUInt(newpointer)
+                                            index += 4
+                                        if not foundone:
+                                            logError("Pointer", toHex(pointer), "not found for string", newsjislog)
+                                            freeranges.pop()
+                            else:
+                                fo.writeZero(endpos - fo.tell())
+                        else:
+                            pos = fi.tell() - 1
+                    fi.seek(pos + 1)
+
+
 # Folders
 def makeFolder(folder, clear=True):
     if clear:
@@ -604,18 +729,18 @@ def readRGB5A1(color):
     r = cc58[color & 0x1f]
     g = cc58[color >> 5 & 0x1f]
     b = cc58[color >> 10 & 0x1f]
-    a = 255 if (color >> 15 & 0x1) == 0 else 0
+    a = 0 if (color >> 15 & 0x1) == 0 else 255
     return (r, g, b, a)
 
 
-def getPaletteIndex(palette, color, fixtrasp=False, starti=0, palsize=-1, checkalpha=False, zerotrasp=True):
-    if color[3] == 0 and zerotrasp:
+def getPaletteIndex(palette, color, fixtransp=False, starti=0, palsize=-1, checkalpha=False, zerotransp=True):
+    if color[3] == 0 and zerotransp:
         return 0
     if palsize == -1:
         palsize = len(palette)
     zeroalpha = -1
     for i in range(starti, starti + palsize):
-        if fixtrasp and i == starti:
+        if fixtransp and i == starti:
             continue
         if palette[i][0] == color[0] and palette[i][1] == color[1] and palette[i][2] == color[2] and (not checkalpha or palette[i][3] == color[3]):
             return i - starti
@@ -656,11 +781,14 @@ def findBestPalette(palettes, colors):
     return disti
 
 
-def drawPalette(pixels, palette, width, ystart=0):
+def drawPalette(pixels, palette, width, ystart=0, transp=True):
     for x in range(len(palette)):
         j = width + ((x % 8) * 5)
         i = ystart + ((x // 8) * 5)
         for j2 in range(5):
             for i2 in range(5):
-                pixels[j + j2, i + i2] = palette[x]
+                color = palette[x]
+                if not transp:
+                    color = (color[0], color[1], color[2], 255)
+                pixels[j + j2, i + i2] = color
     return pixels
