@@ -44,9 +44,11 @@ def repackBIN(binfile, binpatch, cuefile, patchfile=""):
 
 
 # Binary-related functions
-def extractEXE(binranges, detectFunc=common.detectEncodedString, encoding="shift_jis", exein="", exefile="data/exe_output.txt", writepos=False):
+def extractEXE(binrange, detectFunc=common.detectEncodedString, encoding="shift_jis", exein="", exefile="data/exe_output.txt", writepos=False):
     common.logMessage("Extracting EXE to", exefile, "...")
-    strings, positions = extractBinaryStrings(exein, binranges, detectFunc, encoding)
+    if type(binrange) == tuple:
+        binrange = [binrange]
+    strings, positions = common.extractBinaryStrings(exein, binrange, detectFunc, encoding)
     with codecs.open(exefile, "w", "utf-8") as out:
         for i in range(len(strings)):
             if writepos:
@@ -55,29 +57,7 @@ def extractEXE(binranges, detectFunc=common.detectEncodedString, encoding="shift
     common.logMessage("Done! Extracted", len(strings), "lines")
 
 
-def extractBinaryStrings(infile, binranges, func=common.detectEncodedString, encoding="shift_jis"):
-    strings = []
-    positions = []
-    insize = os.path.getsize(infile)
-    with common.Stream(infile, "rb") as f:
-        for binrange in binranges:
-            f.seek(binrange[0])
-            while f.tell() < binrange[1] and f.tell() < insize - 2:
-                pos = f.tell()
-                check = func(f, encoding)
-                if check != "":
-                    if check not in strings:
-                        common.logDebug("Found string at", pos)
-                        strings.append(check)
-                        positions.append([pos])
-                    else:
-                        positions[strings.index(check)].append(pos)
-                    pos = f.tell() - 1
-                f.seek(pos + 1)
-    return strings, positions
-
-
-def repackEXE(binranges, detectFunc=common.detectEncodedString, writeFunc=common.writeEncodedString, encoding="shift_jis", comments="#", exein="", exeout="", exefile="data/exe_input.txt"):
+def repackEXE(binrange, freeranges=None, detectFunc=common.detectEncodedString, writeFunc=common.writeEncodedString, encoding="shift_jis", comments="#", exein="", exeout="", exefile="data/exe_input.txt"):
     if not os.path.isfile(exefile):
         common.logError("Input file", exefile, "not found")
         return False
@@ -88,44 +68,13 @@ def repackEXE(binranges, detectFunc=common.detectEncodedString, writeFunc=common
     with codecs.open(exefile, "r", "utf-8") as bin:
         section = common.getSection(bin, "", comments)
         chartot, transtot = common.getSectionPercentage(section)
-    repackBinaryStrings(section, exein, exeout, binranges, detectFunc, writeFunc, encoding)
+    if type(binrange) == tuple:
+        binrange = [binrange]
+    common.repackBinaryStrings(section, exein, exeout, binrange, freeranges, detectFunc, writeFunc, encoding, 0x8000F800)
     common.logMessage("Done! Translation is at {0:.2f}%".format((100 * transtot) / chartot))
     return True
 
 
-def repackBinaryStrings(section, infile, outfile, binranges, detectFunc=common.detectEncodedString, writeFunc=common.writeEncodedString, encoding="shift_jis"):
-    insize = os.path.getsize(infile)
-    with common.Stream(infile, "rb") as fi:
-        with common.Stream(outfile, "r+b") as fo:
-            for binrange in binranges:
-                fi.seek(binrange[0])
-                while fi.tell() < binrange[1] and fi.tell() < insize - 2:
-                    pos = fi.tell()
-                    check = detectFunc(fi, encoding)
-                    if check != "":
-                        if check in section and section[check][0] != "":
-                            common.logDebug("Replacing string at", pos)
-                            newsjis = section[check][0]
-                            if len(section[check]) > 1:
-                                section[check].pop(0)
-                            if newsjis == "!":
-                                newsjis = ""
-                            newsjislog = newsjis.encode("ascii", "ignore")
-                            fo.seek(pos)
-                            endpos = fi.tell() - 1
-                            newlen = writeFunc(fo, newsjis, endpos - pos + 1, encoding)
-                            fo.seek(-1, 1)
-                            if fo.readByte() != 0:
-                                fo.writeZero(1)
-                            if newlen < 0:
-                                common.logError("String", newsjislog, "is too long.")
-                            else:
-                                fo.writeZero(endpos - fo.tell())
-                        else:
-                            pos = fi.tell() - 1
-                    fi.seek(pos + 1)
-
-
 # Images
 def extractTIM(infolder, outfolder, extensions=".tim", readfunc=None):
     common.makeFolder(outfolder)
@@ -135,8 +84,9 @@ def extractTIM(infolder, outfolder, extensions=".tim", readfunc=None):
         common.logDebug("Processing", file, "...")
         extension = os.path.splitext(file)[1]
         if readfunc is not None:
-            tim, forcepal = readfunc(infolder + file)
+            tim, transp, forcepal = readfunc(infolder + file)
         else:
+            transp = False
             forcepal = -1
             with common.Stream(infolder + file, "rb") as f:
                 tim = readTIM(f)
@@ -145,7 +95,7 @@ def extractTIM(infolder, outfolder, extensions=".tim", readfunc=None):
         # Export img
         common.makeFolders(outfolder + os.path.dirname(file))
         outfile = outfolder + file.replace(extension, ".png")
-        drawTIM(outfile, tim, forcepal)
+        drawTIM(outfile, tim, transp, forcepal)
     common.logMessage("Done! Extracted", len(files), "files")
 
 
@@ -199,7 +149,6 @@ def readTIM(f, forcesize=0):
             clut = []
             for j in range(tim.clutwidth):
                 color = common.readRGB5A1(f.readUShort())
-                color = (color[0], color[1], color[2], 255)
                 clut.append(color)
             tim.cluts.append(clut)
     # Read size
@@ -226,7 +175,6 @@ def readTIM(f, forcesize=0):
                 tim.data.append(f.readByte())
             elif tim.bpp == 16:
                 color = common.readRGB5A1(f.readUShort())
-                color = (color[0], color[1], color[2], 255)
                 tim.data.append(color)
             elif tim.bpp == 24:
                 tim.data.append((f.readByte(), f.readByte(), f.readByte(), 255))
@@ -235,47 +183,60 @@ def readTIM(f, forcesize=0):
     return tim
 
 
-def getUniqueCLUT(tim):
+def getUniqueCLUT(tim, transp=False):
     clut = 0
     # Look for a palette with all different colors to export
     for i in range(len(tim.cluts)):
-        if len(tim.cluts[i]) == len(set(tim.cluts[i])):
+        checkclut = []
+        for color in tim.cluts[i]:
+            if transp:
+                checkclut.append(color)
+            else:
+                checkclut.append((color[0], color[1], color[2]))
+        if len(checkclut) == len(set(checkclut)):
             clut = i
             break
     return clut
 
 
-def drawTIM(outfile, tim, forcepal=-1):
+def drawTIM(outfile, tim, transp=False, forcepal=-1):
     if tim.width == 0 or tim.height == 0:
         return
-    clut = forcepal if forcepal != -1 else getUniqueCLUT(tim)
-    clutsize = 5 * (len(tim.cluts[clut]) // 8)
-    img = Image.new("RGBA", (tim.width + 40, max(tim.height, clutsize)), (0, 0, 0, 0))
+    clutwidth = clutheight = 0
+    if tim.bpp == 4 or tim.bpp == 8:
+        clut = forcepal if forcepal != -1 else getUniqueCLUT(tim, transp)
+        clutwidth = 40
+        clutheight = 5 * (len(tim.cluts[clut]) // 8)
+    img = Image.new("RGBA", (tim.width + clutwidth, max(tim.height, clutheight)), (0, 0, 0, 0))
     pixels = img.load()
     x = 0
     for i in range(tim.height):
         for j in range(tim.width):
             if tim.bpp == 4 or tim.bpp == 8:
-                pixels[j, i] = tim.cluts[clut][tim.data[x]]
+                color = tim.cluts[clut][tim.data[x]]
             else:
-                pixels[j, i] = tim.data[x]
+                color = tim.data[x]
+            if not transp:
+                color = (color[0], color[1], color[2], 255)
+            pixels[j, i] = color
             x += 1
-    pixels = common.drawPalette(pixels, tim.cluts[clut], tim.width, 0)
+    if tim.bpp == 4 or tim.bpp == 8:
+        pixels = common.drawPalette(pixels, tim.cluts[clut], tim.width, 0, transp)
     img.save(outfile, "PNG")
 
 
-def writeTIM(f, tim, infile, forcepal=-1):
+def writeTIM(f, tim, infile, transp=False, forcepal=-1):
     if tim.bpp > 8:
         common.logError("writeTIM bpp", tim.bpp, "not supported")
         return
-    clut = forcepal if forcepal != -1 else getUniqueCLUT(tim)
+    clut = forcepal if forcepal != -1 else getUniqueCLUT(tim, transp)
     img = Image.open(infile)
     img = img.convert("RGBA")
     pixels = img.load()
     f.seek(tim.dataoff)
     for i in range(tim.height):
         for j in range(tim.width):
-            index = common.getPaletteIndex(tim.cluts[clut], pixels[j, i], zerotrasp=False)
+            index = common.getPaletteIndex(tim.cluts[clut], pixels[j, i], checkalpha=transp, zerotransp=False)
             if tim.bpp == 4:
                 f.writeHalf(index)
             else:
